#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "matriz.h"
#include "disco.h"


/* ----------- VARIAVEIS GLOBAIS ---------------- */

/* Descritor de arquivo de entrada 
 * Alterado por: abrir()
 * Usado por: fechar(), getLinhaDados(), reset()
 */
static FILE *fpin;

/* Nome de Arquivo de Entrada
 * Alterado por: abrir()
 * Usado por: salva Result()
 */
static char *arqent = ARQDATA;



/* ------------------ FUNCOES -------------------- */

/**********************************************************************
 *  Funcao abrir
 * 
 *  Acao:       Abre arquivo
 *  Entrada:    file = nome do arquivo.
 *                     Se NULL, assume ARQDATA (vide disco.h)
 *              modo = modo de abertura de arquivo (vide fopen(1))
 *  Retorno:    1 se abriu arquivo
 *              0 caso contrario
 *  Variaveis Globais:  Altera fpin
 **********************************************************************
 */

int abrir (char * file, char *modo)
{
  int ok=1;

  if (file)
    arqent = strdup (file);

  if (!(fpin = fopen (arqent, modo))) {
    ok = 0;
    fprintf (stderr, "Erro na abertura do arq. \"%s\"\n", file);
  }

  return (ok);
}

/**********************************************************************
 *  Funcao fechar
 * 
 *  Acao:       Fecha arquivo
 *  Entrada:    nenhuma
 *  Retorno:    nenhum
 *  Variaveis Globais:  Usa fpin
 **********************************************************************
 */

void fechar ()
{
  fclose (fpin);
}

/**********************************************************************
 *  Funcao reset
 * 
 *  Acao:       Faz "flush" de arquivo de entrada
 *  Entrada:    nenhuma
 *  Retorno:    nenhum
 *  Variaveis Globais:  Usa fpin
 **********************************************************************
 */

void reset ()
{
  /* Flush fpent*/
  while (fgetc(fpin) != '\n');
}


/**********************************************************************
 *  Funcao getLinhaDados
 * 
 *  Acao:       Recupera de arqent linha de valores
 *  Entrada:    nenhuma
 *  Saida:      vetor de doubles (alocado com calloc)
 *  Retorno:    numero de valores na linha/vetor
 *              NULL se chegou ao final de arqent
 *  Variaveis Globais:  Usa fpin
 **********************************************************************
 */

int getLinhaDados (double **dados)
{
  int numval;
  char linha[LINHA_DADOS], *token, *str;
  double *data = calloc (0,sizeof(double));

  token = str = fgets(linha, LINHA_DADOS, fpin);

  for (numval = 0; token; numval++, str = NULL){
    token = strtok(str, " ");
    if (token) {
      data = (double *) realloc ((double *) data, 
				 (numval + 2) * sizeof (double));
      if (sscanf(token, "%lf", &data[numval]) != 1) {
	free(data);
	token = NULL;
	numval = 0;
      }
    }
  }

  if (numval) {
    *dados = data;
    numval--;
  }

  return (numval);

}

/**********************************************************************
 *  Funcao salva
 * 
 *  Acao:       Salva matriz e sua ordem em arquivo
 *  Entrada:    file = nome do arquivo de entrada
 *              matfinal = estrutura com matriz resultado e sua ordem
 *  Retorno:    1 se salvou com sucesso
 *              0 caso contrario
 *  Variaveis Globais:  Usa arqent
 **********************************************************************
 */

int salva (Matrizes *matfinal)
{
  int i, k, ok;
  char *arqsai;
  FILE *fpout;

  /* Gera nome do arquivo de saida = "res-" + nome arq. entrada */
  arqsai = strdup("res-");
  arqsai = realloc (arqsai, strlen(arqsai) + strlen(arqent) + 1);
  arqsai = strcat (arqsai, arqent);

  /* Abre arquivo de saida para anexar e grava resultado */
  if (ok = (fpout = fopen (arqsai, "a")) != NULL) {
    fprintf(fpout, "%d %d\n", matfinal->nordem.linres, matfinal->nordem.colres);
    for (i=0; i < matfinal->nordem.linres; i++) {
      for (k=0; k < matfinal->nordem.colres; k++)
	fprintf(fpout, "%5.2lf ", matfinal->result[i][k]);
      fprintf(fpout, "\n");
    }
    
    fclose (fpout);
  }
  else
    fprintf (stderr, "Erro na abertura do arq. \"%s\"\n", arqsai);

  return (ok);
}

